// defines.c contains all global definitions

// animation defines
// defines for current states
#define moving			1
#define fighting		2
#define blocking		3
#define climbing		4
#define cutscene		5

// defines used in animation states
#define nullframe		-2
#define blend			-1
#define stand			0
#define move			1

// attacks! :D
#define punch			3
#define doublepunch	4
#define kick			5
#define doublekick	6
#define jump			7
#define jumpkick		8
#define block			9

// variable definitions
move_min_z = 0.4;
move_friction = 0.5;

// structs that hold all data objects
// stores animation data
typedef struct {
	var animate;
	var animate2;
	
	int animblend;
	int blendframe;
	
	int previousframe;
	int currentframe;
	int currentblend;
} ANIMATION;

// create global character struct
typedef struct {
	ENTITY* Character;
	ANIMATION anim;
	
	int animationMode;
	int movementMode;
	
	int health;
	
	int gravityOffset;
} CHARACTER;

// create player struct
typedef struct {
	CHARACTER myChar;
	// add rest of player needed stuff here
	int pressedMoveKey;
	
	// create section here that stores key input...
	STRING* keySetup[8]; // 8 keys: 4xmove, 1xjump, 2xattack, 1xblock(?)
	// create array that storeskey input for attack's
	int attackInput[6]; // store combo's of max 6 keys
	int comboCount;
	
	int curHeading;
	VECTOR playerRot;
	
	// variable that checks if player has to be respawned
	int playerReset;
	int playerID;
} PLAYER;

typedef struct {
	PLAYER* playerArray[4]; // contains all players (max 4)
	
	int playerStoreNumber;
	// amount of lives player has
	int lifes[4];
} PLAYERCONTAINER;

// create basic AI struct
typedef struct {
	CHARACTER myChar;
	// add rest of Ai's brain here
	ENTITY* AiTarget;
	
	var damageModifier;
	VECTOR aiRot;
} AI;

// create basic trigger struct for events
typedef struct {
	ENTITY* tempTrigger;
} TRIGGER;
/*
// create basic structure for ladders
typedef struct {
	ENTITY* tempLadderTrigger;
	
	int triggerAvailable;
	
	int moveQuants;
	int modeUpDown;
} LADDER;
*/
// camera setup structure
typedef struct {
	VECTOR* focusPos;
	ENTITY* tempCam;
	
	VECTOR position;
	int updateSpeed;
	int playerCamDist;
	
	int CameraMode;
	int arcZoom;
	int arcZoomMax;
	int arcZoomMin;
	int arcZoomSpeed;
} MOVECAM;

// Global objects
PLAYERCONTAINER* gPlayerList;
MOVECAM* gCamera;

// protoype function
void RunmyChar(PLAYER* tempChar);
void RunmyAi(AI* tempAi);
void updateCamera(MOVECAM* myCam);
void cameraTrigger(TRIGGER* myTrigger);
// void ladderTrigger(LADDER* myLadder);
void levelChange(STRING* levelStr);
void checkPlayerRespawn();

// Global variables
int MultiplayerAmount = 1; // amount of players wanted in-game
int KonamiCodeSet = 0;
int startGame = 0;

// empty prototype function to run other functions from code
void runStringFunction();

// returns smallest angle (used for rotating player on closest rotation angle)
// function borrowed from RUDI open-source project
VECTOR* ang_lerp(VECTOR* resultVec, VECTOR* srcVec, VECTOR* destVec, var blendFactor)
{
	VECTOR ANG_LERP_VEC; // calculation vector
	vec_set(ANG_LERP_VEC, nullvector);
	
	vec_set(resultVec, srcVec);						// Copy source into result
	vec_diff(ANG_LERP_VEC, destVec, resultVec);  // Difference vector

	ANG_LERP_VEC.x = ang(ANG_LERP_VEC.x);			// Adjust angle validity
	ANG_LERP_VEC.y = ang(ANG_LERP_VEC.y);
	ANG_LERP_VEC.z = ang(ANG_LERP_VEC.z);

	vec_scale(ANG_LERP_VEC.x, blendFactor);		// Interpolate the rotation
	vec_add(resultVec, ANG_LERP_VEC);				// Apply interpolated angle

	return(resultVec);									// Return result angle
}

// returns a vector that the camera should focus at
VECTOR* getPlayersFocus() {
	var playerDistance = 0; // gets distance between multiple players
	var currentDistance = 0; // initial set very high; so any new distance will always be below this
	
	VECTOR* focalPos = vector(0,0,0);
	
	var curI = 0;
	var curJ = 0;
	
	if(gPlayerList.playerStoreNumber == 0) { // if no players yet; return zero
		vec_set(focalPos, nullvector);
		return(focalPos);
	} else if(gPlayerList.playerStoreNumber == 1) { // if 1 player; return it's position
		vec_set(focalPos, gPlayerList.playerArray[gPlayerList.playerStoreNumber-1].myChar.Character.x);
		return(focalPos);
	} else { // if more then 1 player; find pair furthest away, and return middle position 
		var i; var j;
		for(i=0; i<gPlayerList.playerStoreNumber; i++) { // get a player
			if(gPlayerList.playerArray[i] != NULL) {
				for(j=0; j<gPlayerList.playerStoreNumber; j++) { // check distance against all other players
					if((gPlayerList.playerArray[j] != NULL) && (j != i)) { // stop if there are no more players on the list, or if player is the same one
						playerDistance = vec_dist(gPlayerList.playerArray[i].myChar.Character, gPlayerList.playerArray[j].myChar.Character); // get distance
						
						if(playerDistance > currentDistance) { // if the distance is bigger
							curI = i;
							curJ = j;
						}
					}
				}
			}
		}
	}
	
	vec_lerp(focalPos, gPlayerList.playerArray[curI].myChar.Character.x, gPlayerList.playerArray[curJ].myChar.Character.x, 0.5);
	return(focalPos);
}

ENTITY* getAIEntity() {
	you = NULL; // if everything else fails, always return NULL
	
	if(gPlayerList.playerStoreNumber == 1) { // if 1 player; return it's position
		you = gPlayerList.playerArray[gPlayerList.playerStoreNumber-1].myChar.Character;
	} else { // if more then 1 player; get random player to attack
		var randNum = integer(random(2));
		you = gPlayerList.playerArray[randNum].myChar.Character;
	}
	
	return(you);
}

// function that makes the impact effect scale and fade
void impactHit() {
	// set alpha
	set(my, TRANSLUCENT | PASSABLE);
	my.alpha = 99;
	
	//set scale
	my.scale_x = 0.3;
	my.scale_y = my.scale_x;
	my.scale_z = my.scale_x;
	
	while(my.alpha  > 0) {
		// increase scale
		my.scale_x += 0.05 * time_step;
		my.scale_y = my.scale_x;
		my.scale_z = my.scale_x;
		
		// decrease alpha
		my.alpha -= 10 * time_step;
		wait(1);
	}
	
	ent_remove(me);
}

// Joozey Mode! lol
//Oldscool
MATERIAL* Oldscool_mat =
{
	effect ="oldscool.fx";
}

function Oldscool_set_Value(Pixelation_x,Pixelation_y)
{
	Oldscool_mat.skill1 = floatv(Pixelation_x);	//100 looks good
	Oldscool_mat.skill2 = floatv(Pixelation_y);	//should be the same as above
}

VIEW* oldSkoolStencil = { 
  material = Oldscool_mat;
  flags = CHILD | PROCESS_TARGET; 
} // view for postprocessing the stencil buffer